//
// Implementazione della Classe Deque. Prima versione.
// Le classi Stack e List sono derivate dalla classe Deque.
//


#include <iostream.h>
#include "Deque1.H"


// Costruttore per il tipo di dato Deque.
Deque::Deque(int size)
{
  if (size < 1) {
    cerr << "Errore: la Struttura dati deve avere almeno un elemento!" << endl;
    exit(-1);
  }

  data = new mytype[size];	// Crea un array di lunghezza assegnata
  
  head = 0;
  len = 0;
  maxlen = size;
}


// Distruttore per il tipo di dato Deque.
Deque::~Deque()
{
  delete[] data;
}


// Restituisce l'elemento in testa alla Deque.
mytype Deque::ReadHead() const
{
  if (IsEmpty())
    Underflow();

  return(data[(head + 1) % maxlen]);
}


// Restituisce l'elemento in testa alla Deque.
mytype Deque::ReadTail() const
{
  if (IsEmpty())
    Underflow();

  return(data[(head + len) % maxlen]);
}


// Rimuove un elemento dalla testa della Deque.
mytype Deque::RemoveHead()
{
  if (IsEmpty())
    Underflow();
  else {
    head = (head + 1) % maxlen;		// Sposto la testa avanti di un posto.
					// Se arrivo in fondo ricomincio da 0.
    len--;
  }

  return(data[head]);		// Restituisco la vecchia testa della Deque.
}


// Rimuove un elemento dalla coda della Deque.
mytype Deque::RemoveTail()
{
  int t = (head + len) % maxlen;

  if (IsEmpty())
    Underflow();
  else
    len--;		// Sposto la coda indietro di un posto.
			// Se arrivo in fondo ricomincio da 0.

  return(data[t]);	// Restituisco la vecchia coda della Deque.
}


// Inserisce l'elemento d in testa alla Deque.
void Deque::InsertHead(const mytype &d)
{
  if (len != maxlen) {
    data[head] = d;
    head = (head - 1 + maxlen) % maxlen;
    len++;
  }
  else Overflow();
}


// Inserisce l'elemento d in coda alla Deque.
void Deque::InsertTail(const mytype &d)
{
  if (len != maxlen) {
    len++;
    data[(head + len) % maxlen] = d;
  }
  else Overflow();
}


// Errore: la Deque e` vuota.
void Deque::Underflow() const
{
  cerr << "Errore: la Struttura dati e` vuota!" << endl;
  exit(-2);
}


// Errore: la Deque e` piena.
void Deque::Overflow() const
{
  cerr << "Errore: la Struttura Dati e` piena!" << endl;
  exit(-3);
}
