//
// Classe per rappresentare una Deque. Prima versione che utilizza un array.
// Le classi Stack e List sono derivate dalla classe Deque.
//

typedef int mytype;			// Il tipo dei nostri dati

class Deque {
  int head;		// Indice della testa
  int len;		// Lunghezza attuale della Deque
  int maxlen;		// Dimensione della Deque
  mytype *data;		// Puntatore agli elementi della Deque

public:
  Deque(int size = 2048);	// Costruttore per Deque
  ~Deque();			// Distruttore per Deque

  mytype ReadHead() const;	// Restituisce l'elemento in testa alla Deque
  mytype ReadTail() const;	// Restituisce l'elemento in coda alla Deque

  mytype RemoveHead();		// Rimuove un elemento dalla testa di Deque
  mytype RemoveTail();		// Rimuove un elemento dalla coda di Deque

  void InsertHead(const mytype &d);   // Inserisce un elemento in testa a Deque
  void InsertTail(const mytype &d);   // Inserisce un elemento in coda a Deque

  void Underflow() const;	// Messaggio d'errore per Deque vuota
  void Overflow() const;	// Messaggio d'errore per Deque Piena

  int IsEmpty()	const		// La Deque e` vuota?
    { return(len == 0); }
};




// La classe Queue e` derivata dalla Deque.
// Deque e` privata, quindi le sue funzioni sono private in Queue.
class Queue: private Deque {
public:
  Queue(int size = 1024):		// Costruttore di una Queue
    Deque(size)
    {};

  void Enqueue(const mytype &d)		// Inserisce un elemento in Queue.
    { Deque::InsertTail(d); }		// L'inserimento e` fatto in coda.

  mytype Dequeue()			// Rimuove un elemento da Queue.
    { return (Deque::RemoveHead()); }	// La rimozione e` fatta dalla testa.

  mytype Read() const			// Legge l'elemento in testa alla Queue.
    { return (Deque::ReadHead()); }

  int IsEmpty()	const			// La Queue e` vuota?
    { return( Deque::IsEmpty()); }
};



// La classe Stack e` derivata dalla Deque.
// Deque e` privata, quindi le sue funzioni sono private in Stack.
class Stack: private Deque {
public:
  Stack(int size = 1024):		// Costruttore di uno Stack.
    Deque(size)
    {};

  void Push(const mytype &d)		// Inserisce un elemento nello Stack.
    { Deque::InsertHead(d); }		// L'inserimento e` fatto in testa.

  mytype Pop()				// Rimuove un elemento dallo Stack.
    { return (Deque::RemoveHead()); }	// La rimozione e` fatta dalla testa.

  mytype Read() const			// Legge l'elemento in testa allo Stack.
    { return (Deque::ReadHead()); }

  int IsEmpty()	const			// Lo Stack e` vuoto?
    { return( Deque::IsEmpty()); }
};
